"""
This module is a part of slcp command line application
and is licensed under the MIT License.
Copyright (c) 2019 Kirill Plotnikov
GitHub: https://github.com/pltnk/selective_copy
PyPi: https://pypi.org/project/slcp/
"""


from slcp.cli import ArgParser
from slcp.process import Handler


def main():
    """Process files according to the command line arguments."""
    parser = ArgParser()
    handler = Handler(parser.args)
    print(handler.message)
    handler.log.logger.info(handler.message)
    handler.process()
    handler.log.logger.info(f"Process finished\n")
    handler.log.close()


if __name__ == "__main__":
    main()



"""
This module is a part of slcp command line application
and is licensed under the MIT License.
Copyright (c) 2019 Kirill Plotnikov
GitHub: https://github.com/pltnk/selective_copy
PyPi: https://pypi.org/project/slcp/
"""


import os
import shutil
import sys
from tkinter.filedialog import askdirectory

from slcp.log import Log


class Handler:
    """File processing handler."""

    def __init__(self, args):
        """
        Initialize Handler object according to arguments given in the command line.
        :param args: argparse.Namespace. Command line arguments.
        """
        self.args = args
        self.source = self.select_source()
        self.destination = self.select_destination()
        self.log = Log(args, self.destination)
        self.todo = self.get_todo()
        self.total = len(self.todo)
        self.action = shutil.move if args.move else shutil.copy
        self.excluded = ", ".join(args.exclude)
        self.processed = 0
        self.message = (
            f'{"Moving" if args.move else "Copying"} '
            f'{self.total} file{"s" if self.total > 1 else ""} '
            f'{"without" if args.invert else "with"} '
            f'{", ".join(args.ext)} extension{"s" if len(args.ext) > 1 else ""} '
            f"from {self.source} to {self.destination}"
            f'{" preserving source folder structure" if args.preserve else ""}'
            f'{f", excluding {self.excluded}" if len(args.exclude) > 0 else ""}'
        )
        try:
            self.check_for_errors()
        except Exception as e:
            self.log.logger.error(f"{e}\n")
            self.log.close()
            sys.exit(e)

    def select_source(self):
        """
        Check if the source path is in the command line arguments,
        if not ask user for input using filedialog.
        :return: str. Source folder path.
        """
        if self.args.srccwd:
            source = os.getcwd()
        else:
            if self.args.source is None:
                print("Choose a source path.")
                source = os.path.normpath(askdirectory())
                print(f"Source path: {source}")
            else:
                source = self.args.source
        return source

    def select_destination(self):
        """
        Check if the destination path is in the command line arguments,
        if not ask user for input using filedialog.
        If the destination path in arguments does not exist create it.
        :return: str. Destination folder path.
        """
        if self.args.dstcwd:
            destination = os.getcwd()
        else:
            if self.args.dest is None:
                print("Choose a destination path.")
                destination = os.path.normpath(askdirectory())
                print(f"Destination path: {destination}")
            else:
                destination = self.args.dest
                if not os.path.exists(destination):
                    os.makedirs(destination)
        return destination

    def get_todo(self):
        """
        Create a to-do list where each sublist represents one file and contains
        source and destination paths for this file.
        :return: list of list of str. To-do list.
        """
        todo_list = []
        try:
            os.chdir(self.source)
            for foldername, _, filenames in os.walk(self.source):
                if self.args.preserve:
                    path = os.path.join(
                        self.destination,
                        f'{"not_" if self.args.invert else ""}'
                        f'{"_".join(self.args.ext).replace(".", "")}'
                        f"_{os.path.basename(self.source)}",
                        os.path.relpath(foldername),
                    )
                for filename in filenames:
                    if (
                        filename.endswith(self.args.ext) ^ self.args.invert
                    ) and filename not in self.args.exclude:
                        if self.args.preserve:
                            todo_list.append(
                                [
                                    os.path.join(foldername, filename),
                                    os.path.join(path, filename),
                                ]
                            )
                        else:
                            todo_list.append(
                                [
                                    os.path.join(foldername, filename),
                                    os.path.join(self.destination, filename),
                                ]
                            )
        except FileNotFoundError:
            pass
        return todo_list

    def check_for_errors(self):
        """Check for errors, raise corresponding Exception if any errors occurred."""
        if not os.path.exists(self.source):
            raise Exception(f"Error: Source path {self.source} does not exist.")
        elif self.total == 0:
            raise Exception(
                f'Error: There are no {", ".join(self.args.ext)} files in {self.source}.'
            )
        elif os.path.commonpath([self.source, self.destination]) == self.source:
            raise Exception(
                f"Error: A destination folder must be outside of source folder. "
                f"Paths given: source - {self.source} | destination - {self.destination}."
            )
        else:
            pass

    def process(self):
        """
        Copy or move files according to source and destination paths
        given in self.todo_list. Each item in this list represents one file.
        item[0] - source, item[1] - destination.
        :return: NoneType.
        """
        self._show_progress_bar()
        for item in self.todo:
            if not os.path.exists(os.path.dirname(item[1])):
                os.makedirs(os.path.dirname(item[1]))
            try:
                if not os.path.exists(item[1]):
                    self.log.logger.info(f"{item[0]}")
                    self.action(item[0], item[1])
                else:
                    new_filename = f"{os.path.basename(os.path.dirname(item[0]))}_{os.path.basename(item[1])}"
                    self.log.logger.info(f"*{item[0]} saving it as {new_filename}")
                    self.action(
                        item[0], os.path.join(os.path.dirname(item[1]), new_filename)
                    )
            except Exception as e:
                self.log.logger.error(
                    f"*Unable to process {item[0]}, an error occurred: {e}"
                )
            self.processed += 1
            self._show_progress_bar()

    def _show_progress_bar(self):
        """
        Print progress bar.
        :return: NoneType.
        """
        try:
            term_width = os.get_terminal_size(0)[0]
        except OSError:
            term_width = 80
        length = term_width - (len(str(self.total)) + 33)
        percent = round(100 * (self.processed / self.total))
        filled = int(length * self.processed // self.total)
        bar = f'|{"=" * filled}{"-" * (length - filled)}|' if term_width > 50 else ""
        suffix = (
            f"Files left: {self.total - self.processed} "
            if self.processed < self.total
            else "Done, log saved"
            if self.args.log
            else "Done           "
        )
        print(f"\rProgress: {bar} {percent}% {suffix}", end="\r", flush=True)
        if self.processed == self.total:
            print()



"""
This module is a part of slcp command line application
and is licensed under the MIT License.
Copyright (c) 2019 Kirill Plotnikov
GitHub: https://github.com/pltnk/selective_copy
PyPi: https://pypi.org/project/slcp/
"""


import os
from argparse import ArgumentParser


class ArgParser:
    """Command line arguments parser."""

    def __init__(self):
        """Parse command line arguments, format given extensions and arguments containing paths."""
        self.parser = ArgumentParser(
            usage="slcp ext [ext ...] [-s SRC] [-d DST] [-sc | -dc] "
            "[-p] [-i] [-m] [-e FILE [FILE ...]] [-l] [-h]",
            description="Copy all files with given extensions from a directory and its subfolders "
            "to another directory. "
            "A destination folder must be outside of a source folder.",
        )
        self.parser.add_argument(
            "ext",
            nargs="+",
            help="one or more extensions, enter without a dot, separate by spaces",
        )
        self.parser.add_argument(
            "-s", "--source", help="source folder path", metavar="SRC"
        )
        self.parser.add_argument(
            "-d", "--dest", help="destination folder path", metavar="DST"
        )
        group = self.parser.add_mutually_exclusive_group()
        group.add_argument(
            "-sc",
            "--srccwd",
            action="store_true",
            help="use current working directory as a source",
        )
        group.add_argument(
            "-dc",
            "--dstcwd",
            action="store_true",
            help="use current working directory as a destination",
        )
        self.parser.add_argument(
            "-p",
            "--preserve",
            action="store_true",
            help="preserve source folder structure",
        )
        self.parser.add_argument(
            "-i",
            "--invert",
            action="store_true",
            help="process only files without given extensions",
        )
        self.parser.add_argument(
            "-m", "--move", action="store_true", help="move files instead of copying"
        )
        self.parser.add_argument(
            "-e",
            "--exclude",
            nargs="+",
            help="exclude certain files from processing",
            metavar="FILE",
        )
        self.parser.add_argument(
            "-l",
            "--log",
            action="store_true",
            help="create and save log to the destination folder",
        )
        self.args = self.parser.parse_args()
        self.args.ext = tuple(set(f".{item}" for item in self.args.ext))
        if self.args.source:
            self.args.source = os.path.normpath(self.args.source.strip())
        if self.args.dest:
            self.args.dest = os.path.normpath(self.args.dest.strip())
        self.args.exclude = (
            tuple(set(self.args.exclude)) if self.args.exclude else tuple()
        )



"""
This module is a part of slcp command line application
and is licensed under the MIT License.
Copyright (c) 2019 Kirill Plotnikov
GitHub: https://github.com/pltnk/selective_copy
PyPi: https://pypi.org/project/slcp/
"""


import logging
import os


class Log:
    """Logging handler."""

    def __init__(self, args, destination):
        """
        Create logger instance and slcp.log file
        in the destination folder if logging is turned on
        in command line arguments. If not create only logger instance.
        :param args: argparse.Namespace. Command line arguments.
        :param destination: str. Destination folder path.
        """
        self.logger = logging.getLogger("slcp")
        self.logger.setLevel(logging.CRITICAL)
        self.log = args.log
        if self.log:
            self.logger.setLevel(logging.INFO)
            fh = logging.FileHandler(
                os.path.join(destination, "slcp.log"), encoding="utf-8"
            )
            formatter = logging.Formatter(
                "%(asctime)s %(message)s", "%d.%m.%Y %H:%M:%S"
            )
            fh.setFormatter(formatter)
            self.logger.addHandler(fh)

    def close(self):
        """
        Close selective_copy.log file and remove it from logger
        file handlers if logging is turned on in command line arguments.
        :return: NoneType.
        """
        if self.log:
            self.logger.handlers[0].close()
            self.logger.handlers = []



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['slcp']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['slcp = slcp.__main__:main']}

setup_kwargs = {
    'name': 'slcp',
    'version': '0.3.0',
    'description': 'Copy all files with given extensions from a directory and its subfolders to another directory.',
    'long_description': "# Selective Copy v0.3.0\n[![Python Version](https://img.shields.io/pypi/pyversions/slcp.svg)](https://www.python.org/downloads/release/python-370/)\n[![PyPi Version](https://img.shields.io/pypi/v/slcp.svg)](https://pypi.org/project/slcp/)\n[![Codacy Badge](https://api.codacy.com/project/badge/Grade/bdde9d33956642129d82d219328ad5cc)](https://www.codacy.com/app/pltnk/selective_copy?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=pltnk/selective_copy&amp;utm_campaign=Badge_Grade)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![License](https://img.shields.io/github/license/pltnk/selective_copy.svg)](https://choosealicense.com/licenses/mit/)\n\nSimple command line application that copies all files with given extensions from a directory and its subfolders to another directory showing progress bar and remaining files counter.\\\nAllows to preserve a source folder structure, to process only files without given extensions, to move files instead of copying, to exclude certain files from processing and to create a log if necessary.\\\nOpens a filedialog if source and/or destination are not given in the command line.\\\nCreates folders in a destination path if they don't exist.\n\n## Installing\n\n```pip install slcp```\n\n## Usage\n\n<pre>\nslcp ext [ext ...] [-s SRC] [-d DST] [-sc | -dc] [-p] [-i] [-m] [-e FILE [FILE ...]] [-l] [-h]\n\nPositional arguments:\next                         One or more extensions of the files to copy. \n                            Enter extensions without a dot and separate by spaces.\n\nOptional arguments:\n-s SRC, --source SRC        Source folder path.\n-d DST, --dest DST          Destination folder path.\n-sc, --srccwd               Use current working directory as a source folder.\n-dc, --dstcwd               Use current working directory as a destination folder.\n-p, --preserve              Preserve source folder structure.\n-i, --invert                Process only files without given extensions.\n-m, --move                  Move files instead of copying, be careful with this option.\n-e FILE [FILE ...],         Exclude one or more files from processing.\n--exclude FILE [FILE ...]   Enter filenames with extensions and separate by spaces.\n-l, --log                   Create and save log to the destination folder.\n-h, --help                  Show this help message and exit.\n</pre>\n\n### Examples\n\n[![asciicast](https://asciinema.org/a/263359.svg)](https://asciinema.org/a/263359?t=2)\n\n## Changelog\n\n### [v0.3.0](https://github.com/pltnk/selective_copy/releases/tag/v0.3.0) - 2019-08-22 \n#### Added\n- [Black](https://github.com/psf/black) code style\n\n#### Changed\n- The code is now divided into separate modules\n- Dots that come with extensions are removed from output folder name. \nThe reason is that folders with a name starting with a dot are considered as hidden on Linux.\n- Log saving indication\n- Name of the logfile\n\n#### Fixed\n- Issue when paths like /home/user/test and /home/user/test2 were considered as nested which lead to an error.\n\n[Compare with v0.2.1](https://github.com/pltnk/selective_copy/compare/v0.2.1...v0.3.0)\n\n### [v0.2.1](https://github.com/pltnk/selective_copy/releases/tag/v0.2.1) - 2019-07-16 \n#### Added\n- Changelog\n\n#### Fixed\n- Readme of the project on [PyPI](https://pypi.org/project/slcp/) and [GitHub](https://github.com/pltnk/selective_copy)\n\n[Compare with v0.2.0](https://github.com/pltnk/selective_copy/compare/v0.2.0...v0.2.1)\n\n### [v0.2.0](https://github.com/pltnk/selective_copy/releases/tag/v0.2.0) - 2019-07-15 \n#### Added\n- Support of processing several extensions at once\n- --invert option\n- --move option\n- --exclude option\n\n[Compare with v0.1.0](https://github.com/pltnk/selective_copy/compare/v0.1.0...v0.2.0)\n\n## License\n\nThis project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details\n\n## Acknowledgments\n\nInspired by the task from [Chapter 9 of Automate the Boring Stuff](https://automatetheboringstuff.com/chapter9/).\n",
    'author': 'Kirill Plotnikov',
    'author_email': 'init@pltnk.dev',
    'url': 'https://github.com/pltnk/selective_copy',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



"""
slcp - command line application that copies all files with given extensions
from a directory and its subfolders to another directory.
Allows to preserve a source folder structure, to process only files without given extensions,
to move files instead of copying, to exclude certain files from processing and to create a log if necessary.
Opens a filedialog if source and/or destination are not given in the command line.
Creates folders in destination if they don't exist.
This project is licensed under the MIT License.
Copyright (c) 2019 Kirill Plotnikov
GitHub: https://github.com/pltnk/selective_copy
PyPi: https://pypi.org/project/slcp/
"""
